/*
  ALCATEL SpeedTouch USB modem microcode extract utility
  Copyright (C) 2001 Benoit PAPILLAULT
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Author   : Benoit PAPILLAULT <benoit.papillault@free.fr>
  Creation : 25/04/2001

  Searching for the microcode is done using the best match for a start and end 
  patterns. This has been tested with windows 1.3 drivers, linux 1.3 drivers
  & linux 1.3.2 drivers (using binary available at the Alcatel website:
  http://www.alcateldsl.com/ ).
*/

/* for ident(1) command */
static char id[] = "@(#) $Id: extract.c,v 1.1 2001/04/25 21:41:46 papillau Exp $";

#include "extract.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>

/*
  in case of error, return NULL.
  else return a pointer to a buffer (allocated using malloc())
  and size contains the buffer size
*/

unsigned char * load_file(const char * file, int * size)
{
	struct stat statbuf;
	int len ;
	unsigned char * buf;
	int fd;

	if (stat(file,&statbuf) != 0)
	{
		perror("stat");
		return NULL;
	}

	len = statbuf.st_size;

	fd = open(file,O_RDONLY);
	if (fd < 0)
	{
		perror("open");
		return NULL;
	}

	buf = (unsigned char *)malloc(len);
	if (buf == NULL)
	{
		perror("malloc");
		close (fd);
		return NULL;
	}

	if (read(fd,buf,len) != len)
	{
		perror("read");
		free (buf);
		close (fd);
		return NULL;
	}

	close(fd);

	*size = len;
	return buf;
}

/*
  search for the longest match of buf2 inside buf1. It returns an offset
  in buf1 for this best match. In case of errors, returns -1
*/

int search_match(unsigned char * buf1, int len1,
				 unsigned char * buf2, int len2)
{
	int best_match = 0; /* number of character matching */
	int best_offset = -1;

	int cur_match, cur_offset;
	int i;

	for (cur_offset=0;cur_offset<=len1-len2;cur_offset++)
	{
		// computing cur_match
		cur_match = 0;
		for (i=0;i<len2;i++)
			if (buf1[cur_offset+i] == buf2[i])
				cur_match ++;

		if (cur_match > best_match)
		{
			best_offset = cur_offset;
			best_match  = cur_match;

			if (best_match == len2)
				break;
		}
	}

	printf("best offset %6d with probability %3d%%\n",best_offset,
		   (100*best_match)/len2);
	return best_offset;
}

/*
  We search for the best match of a start and end sequence.
  Those sequence delimits the microcode. You won't get 100%
  probability, but it will work anyway, for all the microcode I've tested.

  It returns a buffer (unsigned char *) and its size (int *). It nothing
  is found, NULL is returned
*/

unsigned char * extract_microcode(const char *file, int * size)
{
	unsigned char start_buf[] = {
		0x88, 0xf8, 0x00, 0x40, 0x00, 0x00, 0xf2, 0x01, 
		0x4d, 0xea, 0x01, 0xea, 0x0d, 0xc0, 0xa0, 0xe1
	};

	unsigned char stop_buf[] = {
		0x01, 0x00, 0x00, 0x00, 0x40, 0x01, 0x12, 0x08,
		0x04, 0x00, 0x40, 0x00, 0x00, 0x00, 0x01, 0x14
	};

	int start, stop;
	unsigned char * buf1;
	int size1;

	if ((buf1 = load_file(file,&size1)) == NULL)
		return NULL;

	start=search_match(buf1,size1,start_buf,sizeof(start_buf));
	stop =search_match(buf1,size1,stop_buf ,sizeof(stop_buf) );

	if (stop <= start)
		return NULL;

	*size = stop+sizeof(stop_buf) - start;
	return (buf1 + start);
}

#if 0
void usage()
{
	printf("usage: extract file1 file2\n");
	printf("search best matching file1 in file2\n");
	exit (-1);
}

int main(int argc, char *argv[])
{
	const char * file1 = argv[1];
	const char * file2 = argv[2];

	unsigned char * buf1, * buf2;
	int size1, size2;

	int fd;

	if (argc !=2 && argc != 3)
		usage();

	if (argc == 2)
	{
		/* only search for the microcode */

		unsigned char * buf;
		int size;

		buf = extract_microcode(file1,&size);
		if (buf == NULL)
		{
			printf("microcode not found!\n");
			return -1;
		}

		printf("length = %d\n",size);

		switch (size)
		{
		case 526187: printf("microcode linux v1.3 detected\n"); break;
		case 526239: printf("microcode linux v1.3.1 detected\n"); break;
		case 523436: printf("microcode windows v1.3 detected\n"); break;
		}
		
		fd = open("/tmp/microcode.bin",O_RDWR|O_CREAT|O_TRUNC,0666);
		if (fd < 0)
		{
			perror("open");
			return -1;
		}
		
		if (write(fd,buf,size) != size)
		{
			perror("write");
			return -1;
		}
		
		close (fd);
	}
	else
	{
		/* search for file2 inside file1 */

		if ((buf1 = load_file(file1,&size1)) == NULL)
			return -1;

		if ((buf2 = load_file(file2,&size2)) == NULL)
			return -1;
		
		printf("best_match = %d\n",search_match(buf1,size1,buf2,size2));
	}
	return 0;
}
#endif
