/*
  ALCATEL SpeedTouch USB modem monitoring & diagnostics utility.
  Copyright (C) 2001 Benoit PAPILLAULT
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Author   : Benoit PAPILLAULT <benoit.papillault@free.fr>
  Creation : 05/03/2001

  First use of this program is to give diagnostics about your ADSL modem, hence
  the name. Second use to get the ADSL link before starting pppd on it. This
  use UDSL_IOCTL_START & UDSL_IOCTL_STOP from speedtouch.c [available on
  Alcatel web site]. This means that you should do a "modprobe speedtch"
  before lauching this program.

  Don't forget to upload the microcode first, using modem_init.

  24/03/2001: usb_reset() is a dangerous function. Device becomes unusable
  with libusb and does not even appears in /proc/bus/usb/devices. Replug the
  the device to make it work again.
  
  /var/log/messages display
  usb.c: USB disconnect on device 52

  26/03/2001: added test_int1_alt2() to set alternate setting 2 on interface 1.
*/

#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <signal.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <sys/ioctl.h>
#include <time.h>

#include <linux/usbdevice_fs.h>
#include <errno.h>

#include <usb.h>
#include <usbi.h>

static char id[] = "@(#) $Id: modem_diag.c,v 1.3 2001/04/12 22:06:51 papillau Exp $";

/* Alcatel ADSL SpeedTouch USB modem idVendor/idProduct */

#define ST_VENDOR  0x06b9
#define ST_PRODUCT 0x4061
#define ST_CONFIG 0
#define ST_INTERFACE 1

#define CTRL_TIMEOUT 100
#define DATA_TIMEOUT 100

void print_char(unsigned char c)
{
  if (c>=' ' && c<0x7f)
    printf("%c",c);
  /*
    else if (c=='\r' || c=='\n' || c=='\t' || c=='\b')
    printf("%c",c);
  */
  else	printf(".");
}

void dump(unsigned char *buf,int len)
{
  int i,j;
  
  for (i=0;i<len;i+=16)
    {
      for (j=i;j<len && j<i+16;j++)
	printf("%02x ",buf[j]);
      for (;j<i+16;j++)
	printf("   ");
      for (j=i;j<len && j<i+16;j++)
	print_char(buf[j]);
      printf("\n");
    }
}

int get_reference(struct usb_dev_handle *st_handle)
{
  char buf[0xe];
  int i,n;
  
  /* URB 151 : this buffer match (if you reverse one byte with its neighbour)
   the label on the manta box!!! in my case : 3EC18607CAAB06 */

  /* original line */
  n = usb_control_msg(st_handle,0xc0,0x26,0xf7,0,buf,sizeof(buf),CTRL_TIMEOUT);
  if (n<0)
    printf("usb_control_msg: %s\n",usb_strerror());
  else
    {
      printf("Reference     : ");
      //dump(buf,n);
      for (i=0;i<n;i++)
	{
	  int idx = ((i%2)==0) ? i+1 : i-1;
	  if (i==3 || i==12)
	    printf(" ");
	  print_char(buf[idx]);
	}
      printf(" (see under your modem box)\n");
    }

  return n;
}

int my_usb_clear(usb_dev_handle *h,int ep)
{
  int ret;

  ret = usb_control_msg(h,
			USB_RECIP_ENDPOINT,
			USB_REQ_CLEAR_FEATURE,
			0,ep,NULL,0,CTRL_TIMEOUT);
  if (ret < 0)
    printf("CLEAR_FEATURE ep %d: %s\n",ep,usb_strerror());

  ret = usb_resetep(h,ep);
  if (ret < 0)
    printf("usb_resetep: %s\n",usb_strerror());

  return ret;
}

int my_usb_control_msg(usb_dev_handle *h,int request,int value,int size)
{
    unsigned char buf[100];
    int n;

    if (size > sizeof(buf))
      {
	printf("my_usb_control_msg: buffer is too small\n");
	return -1;
      }

    n = usb_control_msg(h,0xc0,request,value,0x00,buf,size,100);
    if (n < 0)
      {
	printf("usb_control_msg: %s\n",usb_strerror());
	return -1;
      }
    /*
    printf("c0 %02x %04x : size %d ",request,value,n);
    dump(buf,n);
    */
    return 0;
}

#define OFFSET_7  0 /* size 1 */
#define OFFSET_b  1 /* size 8 */
#define OFFSET_d  9 /* size 4 */
#define OFFSET_e 13 /* size 1 */
#define OFFSET_f 14 /* size 1 */
#define TOTAL    15

#define SIZE_7 1
#define SIZE_b 8
#define SIZE_d 4
#define SIZE_e 1
#define SIZE_f 1

/* buf MUST BE at least 15 bytes */

int get_state(usb_dev_handle *h, unsigned char *buf)
{
  if (usb_bulk_read(h,0x81,buf,0x10,100) != -1)
    printf("gotcha!\n"); /* always fail? */

  memset(buf,0,TOTAL);
  usb_control_msg(h,0xc0,0x12,0x07,0x00,buf+OFFSET_7,SIZE_7,100);
  usb_control_msg(h,0xc0,0x12,0x0b,0x00,buf+OFFSET_b,SIZE_b,100);
  usb_control_msg(h,0xc0,0x12,0x0d,0x00,buf+OFFSET_d,SIZE_d,100);
  usb_control_msg(h,0xc0,0x01,0x0e,0x00,buf+OFFSET_e,SIZE_e,100);
  usb_control_msg(h,0xc0,0x01,0x0f,0x00,buf+OFFSET_f,SIZE_f,100);

  return 0;
}

/*
  This a magic sequence, totaly unknown to me, it comes from the Windows
  driver ... 
*/

void test_sequence(usb_dev_handle *h)
{
  unsigned char buf[10];
  int n;

  /* URB 147 */
  buf[0] = 0x1c; buf[1] = 0x50;
  n = usb_control_msg(h,0x40,0x01,0x0b,0x00,buf,2,100);
  if (n<0)
    printf("URB147: %s\n",usb_strerror());

  /* URB 148 */
  buf[0] = 0x32; buf[1] = 0x00;
  n = usb_control_msg(h,0x40,0x01,0x02,0x00,buf,2,100);
  if (n<0)
    printf("URB148: %s\n",usb_strerror());

  /* URB 149 */
  buf[0] = 0x01; buf[1] = 0x00; buf[2] = 0x01;
  n = usb_control_msg(h,0x40,0x01,0x03,0x00,buf,3,100);
  if (n<0)
    printf("URB149: %s\n",usb_strerror());

  /* URB 150 */
  buf[0] = 0x01; buf[1] = 0x00; buf[2] = 0x01;
  n = usb_control_msg(h,0x40,0x01,0x04,0x00,buf,3,100);
  if (n<0)
    printf("URB150: %s\n",usb_strerror());
}

/* buf MUST be 15 bytes long */

int print_state(unsigned char *buf, int verbose)
{
  time_t t = time(NULL);
  int down_speed, up_speed;

  if (verbose)
    {
      printf("=== modem state at %.24s ===\n",ctime(&t));
      printf("c0 12 0007 : "); dump(buf+OFFSET_7,SIZE_7);
      printf("c0 12 000b : "); dump(buf+OFFSET_b,SIZE_b);
      printf("c0 12 000d : "); dump(buf+OFFSET_d,SIZE_d);
      printf("c0 01 000e : "); dump(buf+OFFSET_e,SIZE_e);
      printf("c0 01 000f : "); dump(buf+OFFSET_f,SIZE_f);
    }
      
  switch (buf[OFFSET_7])
    {
    case 0x00:
      printf("ADSL line is down\n");
      break;
    case 0x08:
      printf("ADSL line is blocked? try 'rmmod/insmod usb-uhci' ...\n");
      break;
    case 0x10:
      printf("ADSL line is synchronising\n");
      break;
    case 0x20:

      down_speed = buf[OFFSET_b] | (buf[OFFSET_b+1]<<8)
	| (buf[OFFSET_b+2]<<16) | (buf[OFFSET_b+3]<<24);
      up_speed = buf[OFFSET_b+4] | (buf[OFFSET_b+5]<<8)
	| (buf[OFFSET_b+6]<<16) | (buf[OFFSET_b+7]<<24);

      printf("ADSL line is up, downstream at %d kbit/s, "
	     "upstream at %d kbit/s\n",down_speed,up_speed);
      break;
    }
  
  return 0;
}

/*
  send an ioctl code to the speedtouch.c kernel module.
  It is handled by usbdevfs, see the following funtion in
  /usr/src/linux/drivers/usb/devio.c : usbdev_ioctl() & proc_ioctl()
  /usr/src/linux/drivers/usb/speedtouch.c : udsl_usb_ioctl()
*/

int my_usb_ioctl(usb_dev_handle *h,int cmd)
{
  struct usbdevfs_ioctl s;
  int ret;

  s.ifno = 1; 
  /* speedtouch.c manage interface 1 */

  s.ioctl_code = cmd;
  /* should be 1 for UDSL_IOCTL_START and 2 for UDSL_IOCTL_STOP */

  s.data = NULL;

  if ((ret=ioctl(h->fd,USBDEVFS_IOCTL,&s)) < 0)
    {
      perror("ioctl");
      printf("maybe your forget to do 'insmod speedtch'\n");
    }

  return ret;
}

void test_int1_alt2(usb_dev_handle *h)
{
  const int interface = 1;
  const int alternate = 2;

  if (usb_claim_interface(h,interface) < 0)
    printf("usb_claim_interface %d: %s\n",interface,usb_strerror());
  else
    {
      if (usb_set_altinterface(h,alternate) < 0)
	printf("usb_set_altinterface %d: %s\n",alternate,usb_strerror());

      if (usb_release_interface(h,interface) < 0)
	printf("usb_release_interface %d: %s\n",interface,usb_strerror());
    }
}

void usage()
{
  printf("usage: modem_diag [-v] [-m] [-bg] [-s]\n");
  printf("  -v : verbose mode, show extensive diagnostics\n");
  printf("  -m : monitoring mode, keeps watching ADSL connection state\n");
  printf("  -bg: background mode, goes in background when ADSL is up\n");
  printf("  -s : send ioctl & go in bg even if line is ALREADY up\n");
  exit (-1);
}
  
int main(int argc, char *argv[])
{
  struct usb_bus * bus;
  struct usb_device * dev;
  struct usb_device * st_dev = NULL;
  usb_dev_handle * st_handle;
  int i,n;
  int verbose = 0, monitoring = 0, background = 0, restart = 0;
  unsigned char buf[TOTAL+1000], prevbuf[TOTAL];

  for (i=1;i<argc;i++)
    if (strcmp(argv[i],"-v")==0)
      verbose = 1;
    else if (strcmp(argv[i],"-m")==0)
      monitoring = 1;
    else if (strcmp(argv[i],"-bg")==0)
      background = 1;
    else if (strcmp(argv[i],"-s")==0)
      restart = 0;
    else
      usage();

  usb_init();
  usb_find_busses();
  usb_find_devices();

  for (bus=usb_busses;bus;bus=bus->next)
    for (dev=bus->devices;dev;dev=dev->next)
      if (dev->descriptor.idVendor == ST_VENDOR
	  && dev->descriptor.idProduct == ST_PRODUCT)
	st_dev = dev;

  if (st_dev == NULL)
    {
      printf("I'm sorry, I didn't find your ADSL modem!\n");
      return -1;
    }

  printf("Yeah, I found your ADSL modem with VendorID=%04x & ProductID=%04x\n",
	 st_dev->descriptor.idVendor,st_dev->descriptor.idProduct);

  st_handle = usb_open(st_dev);
  if (st_handle == NULL)
    {
      printf("usb_open: %s\n",usb_strerror());
      return -1;
    }

  if (usb_set_configuration(st_handle,1) < 0)
    {
      printf("usb_set_configuration 1: %s\n",usb_strerror());
      return -1;
    }

  /* just for testing */
  test_int1_alt2(st_handle);

  if (usb_claim_interface(st_handle,0) <0)
    {
      printf("usb_claim_interface: %s\n",usb_strerror());
      return -1;
    }
  /*
  n=usb_control_msg(st_handle,0x80,6,0x300|st_dev->descriptor.iManufacturer,0,
		    (char *)&buf,sizeof(buf),CTRL_TIMEOUT);
  if (n > 0)
    {
      printf("Manufacturer  : ");
      for (i=2;i<n;i+=2)
	printf("%c",buf[i]);
      printf("\n");
    }

  n=usb_control_msg(st_handle,0x80,6,0x300|st_dev->descriptor.iProduct,0,
		    (char *)&buf,sizeof(buf),CTRL_TIMEOUT);
  if (n > 0)
    {
      printf("Product       : ");
      for (i=2;i<n;i+=2)
	printf("%c",buf[i]);
      printf("\n");
    }
  
  n=usb_control_msg(st_handle,0x80,6,0x300|st_dev->descriptor.iSerialNumber,0,
		    (char *)&buf,sizeof(buf),CTRL_TIMEOUT);
  if (n > 0)
    {
      printf("Serial number : ");
      for (i=2;i<n;i+=2)
	printf("%c",buf[i]);
      printf("     (also used as MAC address)\n");
    }
  */
  get_reference(st_handle);

  /* some 'reset' on endpoint */

  usb_clear_halt(st_handle,0x81);

  /*  
  {
    int idx, size;
    
    n = usb_bulk_read(st_handle,0x81,buf,10,100);
    if (n<0)
      printf("usb_bulk_read 0x81: %s\n",usb_strerror());
    else
      {
	printf("usb_bulk_read 0x81, len %d : ",n);
	dump(buf,n);
      }
    
    my_usb_control_msg(st_handle,0x12,0x04,2);
    my_usb_control_msg(st_handle,0x12,0x07,1);
    my_usb_control_msg(st_handle,0x12,0x0b,8);
    my_usb_control_msg(st_handle,0x12,0x0d,4);
    my_usb_control_msg(st_handle,0x01,0x0e,1);
    my_usb_control_msg(st_handle,0x01,0x0f,1);
  }
  */

  test_sequence(st_handle);

  /* should start line sync */
  my_usb_control_msg(st_handle,0x12,0x04,2);
  
  get_state(st_handle,prevbuf);
  print_state(prevbuf,verbose);

  if (monitoring)
    {
      for (;;)
	{
	  get_state(st_handle,buf);
	  if (memcmp(buf,prevbuf,TOTAL) != 0)
	    {
	      print_state(buf,verbose);
	  
	      if ( (buf[OFFSET_7] == 0x20 && prevbuf[OFFSET_7] != 0x20)
		   || (buf[OFFSET_7] == 0x20 && restart))
		{
		  printf("ADSL link goes UP\n");
		  my_usb_ioctl(st_handle,1);
		  
		  /* we go in background at this step */
		  if (background)
		    if (fork () != 0)
		      exit (0);
		  
		  restart = 0;
		}
	      
	      if (buf[OFFSET_7] != 0x20
		  && prevbuf[OFFSET_7] == 0x20)
		{
		  printf("ADSL link goes DOWN\n");
		  my_usb_ioctl(st_handle,2);
		}
	      
	      memcpy(prevbuf,buf,TOTAL);
	    }
	}
    }
  
  if (usb_release_interface(st_handle,0) < 0)
    printf("usb_release_interface: %s\n",usb_strerror());
  
  usb_close(st_handle);
  return 0;
}

