/*
  ALCATEL SpeedTouch USB modem microcode upload utility.
  Copyright (C) 2001 Benoit PAPILLAULT
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Author   : Benoit PAPILLAULT <benoit.papillault@free.fr>
  Creation : 05/03/2001
  
  This program uploads the microcode to the ALCATEL SpeedTouch USB modem.
  It uses libusb (http://libusb.sourceforge.net/).
  
  The microcode can be uploaded only once. If the upload is correct,
  ADSL led should be both green & red.
  
  If you try to download the microcode twice, you will get tons of
  timeout errors.

  25/03/2001: added GPL license.
*/

#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <sys/ioctl.h>

#include <usb.h>

static char id[] = "@(#) $Id: modem_init.c,v 1.4 2001/04/12 22:06:51 papillau Exp $";

/* Alcatel ADSL SpeedTouch USB modem idVendor/idProduct */

#define ST_VENDOR  0x06b9
#define ST_PRODUCT 0x4061

#define CTRL_TIMEOUT 1000
#define DATA_TIMEOUT 1000

void print_char(unsigned char c)
{
  if (c>=' ' && c<0x7f)
    printf("%c",c);
  /*
    else if (c=='\r' || c=='\n' || c=='\t' || c=='\b')
    printf("%c",c);
  */
  else	printf(".");
}

void dump(unsigned char *buf,int len)
{
  int i,j;
  
  for (i=0;i<len;i+=16)
    {
      for (j=i;j<len && j<i+16;j++)
	printf("%02x ",buf[j]);
      for (;j<i+16;j++)
	printf("   ");
      for (j=i;j<len && j<i+16;j++)
	print_char(buf[j]);
      printf("\n");
    }
}

unsigned char data1[] = {
  0x88, 0xf8, 0x00, 0x00, 0x00, 0x00, 0xf2, 0x01, 
  0x0d, 0x00, 0x00, 0xea, 0x0c, 0x00, 0x00, 0xea, 
  0x0b, 0x00, 0x00, 0xea, 0x0a, 0x00, 0x00, 0xea, 
  0x09, 0x00, 0x00, 0xea, 0x08, 0x00, 0x00, 0xea, 
  0xa4, 0x00, 0x00, 0xea, 0x06, 0x00, 0x00, 0xea, 
  0x32, 0x11, 0xa0, 0xe3, 0x5c, 0x03, 0x9f, 0xe5, 
  0x00, 0x00, 0x81, 0xe5, 0x58, 0x13, 0x9f, 0xe5, 
  0x00, 0x00, 0xa0, 0xe3, 0x00, 0x00, 0x81, 0xe5, 
  0xf8, 0xff, 0xff, 0xea, 0x01, 0x00, 0xa0, 0xe3, 
  0x48, 0x13, 0x9f, 0xe5, 0x00, 0x00, 0x81, 0xe5, 
  0x3c, 0x13, 0x9f, 0xe5, 0x04, 0x00, 0xa0, 0xe3, 
  0x00, 0x00, 0x81, 0xe5, 0xd1, 0x00, 0xa0, 0xe3, 
  0x00, 0xf0, 0x29, 0xe1, 0x01, 0xd9, 0xa0, 0xe3, 
  0xd2, 0x00, 0xa0, 0xe3, 0x00, 0xf0, 0x29, 0xe1, 
  0x24, 0xd3, 0x9f, 0xe5, 0xd3, 0x00, 0xa0, 0xe3, 
  0x00, 0xf0, 0x29, 0xe1, 0x1c, 0xd3, 0x9f, 0xe5, 
  0x00, 0x00, 0xa0, 0xe3, 0x00, 0x10, 0xa0, 0xe3, 
  0xb0, 0x02, 0x9f, 0xe5, 0x08, 0x00, 0x80, 0xe2, 
  0xbc, 0x12, 0x9f, 0xe5, 0x00, 0x10, 0x80, 0xe5, 
  0x5f, 0x10, 0xe0, 0xe3, 0x5d, 0x30, 0xe0, 0xe3, 
  0x5c, 0x50, 0xe0, 0xe3, 0x5b, 0x70, 0xe0, 0xe3, 
  0xac, 0x02, 0x9f, 0xe5, 0xa4, 0x22, 0x9f, 0xe5, 
  0xa8, 0x42, 0x9f, 0xe5, 0xa8, 0x62, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x00, 0x30, 0x82, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x70, 0x86, 0xe5, 0x54, 0x02, 0x9f, 0xe5, 
  0x08, 0x00, 0x80, 0xe2, 0x5c, 0x12, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x6c, 0x02, 0x9f, 0xe5, 
  0x6c, 0x12, 0x9f, 0xe5, 0x00, 0x10, 0x80, 0xe5, 
  0x00, 0x10, 0xa0, 0xe3, 0x98, 0x02, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x94, 0x02, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x01, 0x09, 0xa0, 0xe3, 
  0x00, 0x10, 0xa0, 0xe3, 0x02, 0x26, 0xa0, 0xe3, 
  0x04, 0x30, 0xa0, 0xe3, 0x03, 0x10, 0x80, 0xe6, 
  0x02, 0x00, 0x50, 0xe1, 0xfc, 0xff, 0xff, 0x1a, 
  0x7f, 0x19, 0xa0, 0xe3, 0x1e, 0x30, 0xa0, 0xe3, 
  0x03, 0x3b, 0x83, 0xe2, 0x00, 0x00, 0xa0, 0xe3, 
  0x00, 0x2a, 0x83, 0xe1, 0x01, 0x00, 0x80, 0xe2, 
  0x14, 0x00, 0x50, 0xe3, 0x04, 0x20, 0x81, 0xe4, 
  0xfa, 0xff, 0xff, 0xba, 0x02, 0xe0, 0xa0, 0xe3, 
  0x03, 0xeb, 0x8e, 0xe2, 0x0e, 0x20, 0xa0, 0xe1, 
  0x18, 0x00, 0x50, 0xe3, 0x04, 0x00, 0x00, 0xaa, 
  0x00, 0xca, 0x82, 0xe1, 0x01, 0x00, 0x80, 0xe2, 
  0x18, 0x00, 0x50, 0xe3, 0x04, 0xc0, 0x81, 0xe4, 
  0xfa, 0xff, 0xff, 0xba, 0x40, 0x00, 0x50, 0xe3, 
  0x04, 0x00, 0x00, 0xaa, 0x00, 0x2a, 0x83, 0xe1, 
  0x01, 0x00, 0x80, 0xe2, 0x40, 0x00, 0x50, 0xe3, 
  0x04, 0x20, 0x81, 0xe4, 0xfa, 0xff, 0xff, 0xba, 
  0x01, 0x0a, 0x50, 0xe3, 0x04, 0x00, 0x00, 0xaa, 
  0x00, 0x2a, 0x8e, 0xe1, 0x01, 0x00, 0x80, 0xe2, 
  0x01, 0x0a, 0x50, 0xe3, 0x04, 0x20, 0x81, 0xe4, 
  0xfa, 0xff, 0xff, 0xba, 0x7f, 0x09, 0xa0, 0xe3, 
  0x03, 0x40, 0xa0, 0xe3, 0x10, 0x0f, 0x02, 0xee, 
  0x10, 0x4f, 0x03, 0xee, 0x10, 0x0f, 0x07, 0xee, 
  0x10, 0x0f, 0x05, 0xee, 0x3d, 0x00, 0xa0, 0xe3, 
  0x10, 0x0f, 0x01, 0xee, 0x01, 0x73, 0xa0, 0xe3, 
  0x04, 0x00, 0xa0, 0xe3, 0x05, 0x00, 0xc7, 0xe5, 
  0x01, 0x00, 0xa0, 0xe3, 0x06, 0x00, 0xc7, 0xe5, 
  0x01, 0x00, 0xa0, 0xe3, 0x07, 0x00, 0xc7, 0xe5, 
  0x03, 0x00, 0xa0, 0xe3, 0x08, 0x00, 0xc7, 0xe5, 
  0x00, 0x00, 0x40, 0x01, 0x12, 0x88, 0xd4, 0xf2, 
  0x01, 0x00, 0x00, 0xce, 0x01, 0xa0, 0xe3, 0x09, 
  0x00, 0xc7, 0xe5, 0x80, 0x00, 0xa0, 0xe3, 0x01, 
  0x00, 0xc7, 0xe5, 0x53, 0x00, 0xa0, 0xe3, 0x00, 
  0xf0, 0x29, 0xe1, 0x94, 0x11, 0x9f, 0xe5, 0x21, 
  0x03, 0xa0, 0xe3, 0x00, 0x10, 0x80, 0xe5, 0x10, 
  0x10, 0xa0, 0xe3, 0x22, 0x01, 0xa0, 0xe3, 0x00, 
  0x10, 0x80, 0xe5, 0x04, 0x00, 0x8f, 0xe2, 0x01, 
  0x00, 0x80, 0xe2, 0x10, 0xff, 0x2f, 0xe1, 0x5e, 
  0x4f, 0xb8, 0x78, 0x40, 0x1e, 0xfc, 0xd1, 0x38, 
  0x78, 0xf9, 0x7a, 0x09, 0x02, 0xbc, 0x7a, 0x21, 
  0x43, 0x09, 0x02, 0x7c, 0x7a, 0x21, 0x43, 0x09, 
  0x02, 0x3c, 0x7a, 0x21, 0x43, 0x11, 0x38, 0x01, 
  0x28, 0x2d, 0xdb, 0x03, 0x28, 0x2b, 0xdc, 0x80, 
  0x00, 0x55, 0x4c, 0x24, 0x58, 0xa7, 0x46, 0x46, 
  0x4b, 0x0e, 0x33, 0x7a, 0x7b, 0x12, 0x02, 0x3c, 
  0x7b, 0x22, 0x43, 0x12, 0xd0, 0x14, 0x1c, 0xf4, 
  0x3c, 0xff, 0x3c, 0x0e, 0xd5, 0x40, 0x07, 0x02, 
  0xd0, 0x1c, 0x1c, 0x0b, 0x1c, 0x21, 0x1c, 0x52, 
  0x1e, 0x8c, 0x5c, 0x9c, 0x54, 0xfb, 0xd1, 0x3b, 
  0xa4, 0x14, 0x23, 0x23, 0x60, 0xc0, 0x0f, 0x64, 
  0x30, 0x00, 0xe0, 0x66, 0x20, 0x78, 0x70, 0xcb, 
  0xe7, 0x01, 0xa2, 0x10, 0x47, 0x00, 0x00, 0x01, 
  0xf0, 0xa0, 0xe1, 0x04, 0x20, 0x8f, 0xe2, 0x01, 
  0x20, 0x82, 0xe2, 0x12, 0xff, 0x2f, 0xe1, 0x60, 
  0x20, 0xf0, 0xe7, 0x04, 0xe0, 0x4e, 0xe2, 0x00, 
  0x40, 0x2d, 0xe9, 0x00, 0xe0, 0x4f, 0xe1, 0x07, 
  0x40, 0x2d, 0xe9, 0xa8, 0x10, 0x8f, 0xe2, 0x00, 
  0x00, 0x91, 0xe5, 0x01, 0x00, 0x20, 0xe2, 0x00, 
  0x00, 0x81, 0xe5, 0x00, 0x00, 0x50, 0xe3, 0x03, 
  0x00, 0x00, 0x0a, 0x0d, 0x10, 0xa0, 0xe3, 0xcc, 
  0x20, 0x9f, 0xe5, 0x00, 0x10, 0x82, 0xe5, 0x02, 
  0x00, 0x00, 0xea, 0x0f, 0x10, 0xa0, 0xe3, 0xbc, 
  0x20, 0x9f, 0xe5, 0x00, 0x10, 0x82, 0xe5, 0x01, 
  0x00, 0xa0, 0xe3, 0x90, 0x10, 0x9f, 0xe5, 0x00, 
  0x00, 0x81, 0xe5, 0x6c, 0x10, 0x8f, 0xe2, 0x00, 
  0x00, 0x91, 0xe5, 0x01, 0x00, 0x40, 0xe2, 0x00, 
  0x00, 0x81, 0xe5, 0x00, 0x00, 0x50, 0xe3, 0x02, 
  0x00, 0x00, 0x1a, 0x94, 0x00, 0x9f, 0xe5, 0x94, 
  0x10, 0x9f, 0xe5, 0x00, 0x00, 0x81, 0xe5, 0x90, 
  0x20, 0x9f, 0xe5, 0x00, 0x10, 0x92, 0xe5, 0x07, 
  0x40, 0xbd, 0xe8, 0x0e, 0xf0, 0x69, 0xe1, 0x00, 
  0x80, 0xfd, 0xe8, 0x00, 0x00, 0x00, 0x8c, 0x20, 
  0x00, 0x00, 0x88, 0x50, 0x00, 0x00, 0x01, 0x02, 
  0x00, 0x00, 0x00, 0xf4, 0x00, 0x00, 0x00, 0x39, 
  0x00, 0x00, 0x00, 0x10, 0x40, 0x08, 0x00, 0x00, 
  0x40, 0x00, 0x00, 0x20, 0x40, 0x00, 0x00, 0x30, 
  0x4e, 0x00, 0x00, 0x00, 0x00, 0x00, 0xa0, 0x15, 
  0x80, 0x00, 0x00, 0x15, 0x2f, 0x8c, 0x9b, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x78, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x5d, 
  0x02, 0x00, 0x00, 0x5d, 0x02, 0x00, 0x00, 0x97, 
  0x02, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x40, 
  0x00, 0x40, 0x01, 0x64, 0x00, 0x50, 0x01, 0x70, 
  0x3e, 0x00, 0x00, 0xe0, 0x3c, 0x00, 0x00, 0x20, 
  0x00, 0x00, 0x90, 0x20, 0x00, 0x00, 0xa4, 0x85, 
  0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x78, 
  0x03, 0x00, 0x00, 0x58, 0x00, 0x50, 0x01, 0x70, 
  0xc3, 0x00, 0x00, 0x60, 0x00, 0x50, 0x01, 0x04, 
  0x00, 0x00, 0x84, 0x40, 0x01, 0x12, 0x08, 0x04, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x14
};

int my_usb_clear(usb_dev_handle *h, int ep)
{
  int n;
  
  n = usb_control_msg(h,
		      USB_RECIP_ENDPOINT,
		      USB_REQ_CLEAR_FEATURE,
		      0,ep,NULL,0,CTRL_TIMEOUT);
  if (n < 0)
    printf("CLEAR_FEATURE: %s\n",usb_strerror());

  n = usb_resetep(h,ep);
  if (n < 0)
    printf("usb_resetep: %s\n",usb_strerror());

  return 0;
}

void usage()
{
  printf("usage: modem_init [-d] microcode.bin\n");
  exit (-1);
}

int main(int argc, char *argv[])
{
  int fdusb;

  usb_dev_handle * st_handle ;
  FILE * microcode;
  int verbose = 0;
  const char * filename = NULL;
  int i;
  int n;
  char buf[0x1000];

  
  /*
    You should have insmod'd "usbcore" & "usb-uhci" according to
    your USB host controller. At this stage, both green leds should be ON
  */

  /* check args for "-d" option & the microcode filename */

  for (i=1;i<argc;i++)
    {
      if (strcmp(argv[i],"-d")==0)
	verbose = 1;
      else if (filename == NULL)
	filename = argv[i];
      else
	usage();
    }

  if (filename == NULL)
    usage();

  fdusb = usbfs_search("/proc/bus/usb",ST_VENDOR,ST_PRODUCT);
  if (fdusb < 0)
    {
      printf("I'm sorry, I didn't find your ADSL modem!\n");
      return -1;
    }

  printf("Gotcha, I found your ADSL ALCATEL SpeedTouch USB modem!\n");

  if (usbfs_set_configuration(fdusb,1) < 0)
    {
      perror("usbfs_set_configuration 1");
      return -1;
    }

  if (usbfs_claimintf(fdusb,2) <0)
    {
      perror("usbfs_claimintf 2");
      return -1;
    }

  /* URB 8 : both leds are static green */
  n = usb_bulk_write(st_handle,0x5,data1,sizeof(data1),DATA_TIMEOUT);
  if (n < 0)
    printf("usb_bulk_write: %s\n",usb_strerror());
  else
    printf("BLOCK1 : %6d bytes   uploaded : OK\n",sizeof(data1));

  /* after usb_bulk_write :
     + USB  led : blinking green
     + ADSL led : off
  */

  /* URB 9 & 10 */
  my_usb_clear(st_handle,0x85);

  /* URB 11 */
  n = usb_bulk_read(st_handle,0x85,buf,0x1ff /* !!! */,DATA_TIMEOUT);
  if (n < 0)
    printf("usb_bulk_read: %s\n",usb_strerror());
  else
    {
      printf("BLOCK2 : %6d bytes downloaded : OK\n",n);
      if (verbose) dump(buf,n);
    }


  /* URB 12 to 139 */

  microcode = fopen(filename,"r");
  if (microcode != NULL)
    {
      unsigned char mm[0x1000];
      int total = 0, len;

      while ((len = fread(mm,1,sizeof(mm),microcode)) > 0)
	{
	  n = usb_bulk_write(st_handle,0x05,mm,len,DATA_TIMEOUT);
	  if (n < 0)
	    printf("usb_bulk_write: %s\n",usb_strerror());
	  else
	    total += len;
	}

      fclose (microcode);
      printf("BLOCK3 : %6d bytes   uploaded : OK\n",total);
      /* before downloading microcode :
	 + USB  led : blinking
	 + ADSL led : off

	 during downloading:
	 + USB  led : blinking green
	 + ADSL led : off

	 after downloading microcode :
	 + USB  led : static green
	 + ADSL led : static red
      */
    }

  /* URB 140 & 141 */
  my_usb_clear(st_handle,0x85);
  
  /* URB 142 */
  n = usb_bulk_read(st_handle,0x85,buf,0x1ff /* !!! */,DATA_TIMEOUT);
  if (n < 0)
    printf("usb_bulk_read: %s\n",usb_strerror());
  else
    {
      printf("BLOCK4 : %6d bytes downloaded : OK\n",n);
      if (verbose) dump(buf,n);
    }

  usb_release_interface(st_handle,2);
  usb_close(st_handle);
  return 0;
}

