/*
  Author : Benoit PAPILLAULT <benoit.papillault@free.fr>
  Creation : 26/03/2001

  Goals : trying to read from an endpoint with no timeout, using usbdevfs.
  Warning : libusb can't be used there.

  27/03/2001: yes, it works. I manage to use SUBMIT_URB and REAPURB
  to read from an usb endpoint. I the URB size is 8, the reading is the
  fastest. why? anyway, it works. And the signal stuff too.
*/

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <signal.h>

#include <linux/usb.h>
#include <linux/usbdevice_fs.h>
#include <asm/page.h>

/* Device descriptor */
struct usb_device_descriptor {
	__u8  bLength;
	__u8  bDescriptorType;
	__u16 bcdUSB;
	__u8  bDeviceClass;
	__u8  bDeviceSubClass;
	__u8  bDeviceProtocol;
	__u8  bMaxPacketSize0;
	__u16 idVendor;
	__u16 idProduct;
	__u16 bcdDevice;
	__u8  iManufacturer;
	__u8  iProduct;
	__u8  iSerialNumber;
	__u8  bNumConfigurations;
} __attribute__ ((packed));

/* try to open the file and get USB device information,
   if it's ok, check if it matches vendorID & productID and return
   a usable file descriptor in this case. Else returns -1.
*/

int test_file(const char *path, int vendorID, int productID)
{
  int fd;
  struct usb_device_descriptor desc;
  
  fd = open(path,O_RDWR);
  if (fd == -1)
    {
      perror(path);
      return -1;
    }
  
  if (read(fd,&desc,sizeof(desc)) == sizeof(desc))
    {
      /* great, we read something */
      
      /* check, it match the correct structure */
      
      if (desc.bLength == sizeof(desc))
	{
	  
	  printf("=== %s ===\n",path);
	  printf("  bLength         = %u\n",desc.bLength);
	  printf("  bDescriptorType = %u\n",desc.bDescriptorType);
	  printf("  bcdUSB          = %04x\n",desc.bcdUSB);
	  printf("  idVendor        = %04x\n",desc.idVendor);
	  printf("  idProduct       = %04x\n",desc.idProduct);
	  printf("  bcdDevice       = %04x\n",desc.bcdDevice);
	  
	  if (vendorID == desc.idVendor
	      && productID == desc.idProduct)
	    return fd;
	}
    }
  
  close(fd);
  return -1;
}

/* search for a vendorID, productID. return -1 if search failed
   or a usable file descriptor
*/

int scan_dir(const char *path, int vendorID, int productID)
{
  int result = -1;
  DIR * dir;
  struct dirent * dirp;
  
  dir = opendir(path);
  if (dir == NULL)
    {
      perror(path);
      return -1;
    }
  
  while ((dirp=readdir(dir)) != NULL)
    {
      char file[PATH_MAX+1];
      struct stat statbuf;
      
      if (strcmp(dirp->d_name,".")==0)
	continue;
      if (strcmp(dirp->d_name,"..")==0)
	continue;
      
      sprintf(file,"%s/%s",path,dirp->d_name);
      
      if (stat(file,&statbuf)!=0)
	{
	  perror(file);
	  continue;
	}
      
      if (S_ISDIR(statbuf.st_mode))
	{
	  if ((result = scan_dir(file,vendorID,productID)) < 0)
	    continue;
	  else
	    break;
	}
      
      if (S_ISREG(statbuf.st_mode))
	{
	  if ((result=test_file(file,vendorID,productID)) < 0)
	    continue;
	  else
	    break;
	}
    }

  closedir(dir);
  return result;
}

/* ====================== START OF LIBRARY ==================================*/

/* USBDEVFS_CONTROL => proc_control() => usb_control_msg() */

int myusb_control_msg(int fd,int ep,int request,int requesttype,int value,
		      int idx,void * buf,int len, int timeout)
{
  int ret;
  struct usbdevfs_ctrltransfer ctrl;

  ctrl.requesttype = requesttype;
  ctrl.request     = request;
  ctrl.value       = value;
  ctrl.index       = idx;
  ctrl.length      = len;
  ctrl.timeout     = timeout;
  ctrl.data        = buf;

  ret = ioctl(fd,USBDEVFS_CONTROL,&ctrl);
  return ret;
}

/* USBDEVFS_BULK => proc_bulk() => usb_bulk_msg() */

/* from "devio.c", bulk.len <= PAGE_SIZE, else -EINVAL is returned.
   else the real length read is returned (see proc_bulk()).

   myusb_bulk_msg() reads/write what is available. timeout is specified in
   ms. the number of bytes read/written is returned or -1 if an error occurs.
*/

int myusb_bulk_msg(int fd,int ep,unsigned char *buf,int size, int timeout)
{
  struct usbdevfs_bulktransfer bulk;
  int ret, received = 0;

  do
    {
      bulk.ep      = ep;
      bulk.len     = size;
      if (bulk.len > PAGE_SIZE)
	bulk.len = PAGE_SIZE;
      bulk.timeout = timeout;
      bulk.data    = buf;
      
      ret = ioctl(fd,USBDEVFS_BULK,&bulk);
      if (ret < 0)
	return ret;

      buf  += ret;
      size -= ret;

      received += ret;
    }
  while (ret==bulk.len && size>0);

  return received;
}

/*
  USBDEVFS_RESETEP
  USBDEVFS_RESET
  USBDEVFS_CLEAR_HALT
*/

/* USBDEVFS_GETDRIVER => proc_getdriver() => nothing */

/* USBDEVFS_CONNECTINFO => proc_connectinfo() => nothing */

/* USBDEVFS_SETINTERFACE => proc_setintf() => usb_set_interface() */

int myusb_set_interface(int fd, int interface, int altsetting)
{
  struct usbdevfs_setinterface setintf;
  int ret;

  setintf.interface = interface;
  setintf.altsetting = altsetting;

  ret = ioctl(fd,USBDEVFS_SETINTERFACE,&setintf);
  return ret;
}

/* USBDEVFS_SETCONFIGURATION => proc_setconfig => usb_set_configuration */

int myusb_set_configuration(int fd,int config)
{
  int ret;

  ret = ioctl(fd,USBDEVFS_SETCONFIGURATION,&config);
  return ret;
}

/* USBDEVFS_SUBMITURB => proc_submiturb() => usb_submit_urb() */

int myusb_submit_urb(int fd, struct usbdevfs_urb *urb)
{
  int ret;

  ret = ioctl(fd,USBDEVFS_SUBMITURB,urb);
  return ret;
}

/* USBDEVFS_DISCARDURB */

/* USBDEVFS_REAPURB => proc_reapurb() => nothing.

   On return urb points to a pointer of type struct usbdevfs_urb.
   This pointer is the same one, passed to myusb_submit_urb()
*/

int myusb_reapurb(int fd,struct usbdevfs_urb **purb)
{
  int ret;

  ret = ioctl(fd,USBDEVFS_REAPURB,purb);
  return ret;
}

/*
  USBDEVFS_REAPURBNDELAY
  USVDEVFS_DISCSIGNAL
  USBDEVFS_CLAIMINTERFACE
  USBDEVFS_RELEASEINTERFACE
  USBDEVFS_IOCTL
*/

/* ======================= END OF LIBRARY ===================================*/

/* === BEGIN OF UTILITY === */

int USB_read_1(int fd,void * buf, int size)
{
  int ret;
  struct timeval debut, fin;

  gettimeofday(&debut,NULL);
  ret = myusb_bulk_msg(fd,0x85,buf,size,50);
  gettimeofday(&fin,NULL);

  printf("USB_read (%d) = %d in %f ms\n",size,ret,
	 (double)(fin.tv_sec-debut.tv_sec)*1000.0
	 +(double)(fin.tv_usec-debut.tv_usec)/1000.0);

  return ret;
}

void USB_bulk_URB(struct usbdevfs_urb * urb, int ep, void *buf, int len)
{
  memset(urb,0,sizeof(*urb));
  /*
    unsigned char type;
    unsigned char endpoint;
    int status;
    unsigned int flags;
    void *buffer;
    int buffer_length;
    int actual_length;
    int start_frame;
    int number_of_packets;
    int error_count;
    unsigned int signr;
    void *usercontext;
    struct usbdevfs_iso_packet_desc iso_frame_desc[0];
  */

  urb->type = USBDEVFS_URB_TYPE_BULK;
  urb->endpoint = ep;
  urb->flags = 0;
  urb->buffer = buf;
  urb->buffer_length = len;
  urb->signr = 0; /*SIGRTMIN;*/

  /* signr : 0 for no signal, or a signal between SIGRTMIN & SIGRTMAX */
}

int USB_read_small(int fd,void * buf, size_t count)
{
  int ret;
  struct timeval debut, fin;
  struct usbdevfs_urb my_urb, *purb;

  if (count > PAGE_SIZE)
    count = PAGE_SIZE;

  USB_bulk_URB(&my_urb,0x87,buf,count);

  gettimeofday(&debut,NULL);

  ret = ioctl(fd,USBDEVFS_SUBMITURB,&my_urb);
  if (ret < 0)
    perror("ioctl USBDEVFS_SUBMITURB");
  else
    {
      ret = ioctl(fd,USBDEVFS_REAPURB,&purb);
      if (ret < 0)
	perror("USBDEVFS_REAPURB");
      else
	{
	  printf("&my_urb = %p, purb = %p\n",&my_urb,purb);
	  ret = my_urb.actual_length;
	}
    }

  gettimeofday(&fin,NULL);

  printf("USB_read (%d) = %d in %f ms\n",count,ret,
	 (double)(fin.tv_sec-debut.tv_sec)*1000.0
	 +(double)(fin.tv_usec-debut.tv_usec)/1000.0);
  
  return ret;
}

int USB_read(int fd,void *buf, size_t count)
{
  int ret, received = 0, small_count;

  do
    {
      small_count = count;
      if (small_count > 8)
	small_count = 8;

      ret = USB_read_small(fd,buf,count);
      if (ret < 0)
	return ret;

      buf   += ret;
      count -= ret;

      received += ret;
    }
  while (ret == small_count && count>0);

  return received;
}

/* === END OF UTILITY === */

#define ST_VENDOR  0x06b9
#define ST_PRODUCT 0x4061

unsigned char data1[] = {
  0x88, 0xf8, 0x00, 0x00, 0x00, 0x00, 0xf2, 0x01, 
  0x0d, 0x00, 0x00, 0xea, 0x0c, 0x00, 0x00, 0xea, 
  0x0b, 0x00, 0x00, 0xea, 0x0a, 0x00, 0x00, 0xea, 
  0x09, 0x00, 0x00, 0xea, 0x08, 0x00, 0x00, 0xea, 
  0xa4, 0x00, 0x00, 0xea, 0x06, 0x00, 0x00, 0xea, 
  0x32, 0x11, 0xa0, 0xe3, 0x5c, 0x03, 0x9f, 0xe5, 
  0x00, 0x00, 0x81, 0xe5, 0x58, 0x13, 0x9f, 0xe5, 
  0x00, 0x00, 0xa0, 0xe3, 0x00, 0x00, 0x81, 0xe5, 
  0xf8, 0xff, 0xff, 0xea, 0x01, 0x00, 0xa0, 0xe3, 
  0x48, 0x13, 0x9f, 0xe5, 0x00, 0x00, 0x81, 0xe5, 
  0x3c, 0x13, 0x9f, 0xe5, 0x04, 0x00, 0xa0, 0xe3, 
  0x00, 0x00, 0x81, 0xe5, 0xd1, 0x00, 0xa0, 0xe3, 
  0x00, 0xf0, 0x29, 0xe1, 0x01, 0xd9, 0xa0, 0xe3, 
  0xd2, 0x00, 0xa0, 0xe3, 0x00, 0xf0, 0x29, 0xe1, 
  0x24, 0xd3, 0x9f, 0xe5, 0xd3, 0x00, 0xa0, 0xe3, 
  0x00, 0xf0, 0x29, 0xe1, 0x1c, 0xd3, 0x9f, 0xe5, 
  0x00, 0x00, 0xa0, 0xe3, 0x00, 0x10, 0xa0, 0xe3, 
  0xb0, 0x02, 0x9f, 0xe5, 0x08, 0x00, 0x80, 0xe2, 
  0xbc, 0x12, 0x9f, 0xe5, 0x00, 0x10, 0x80, 0xe5, 
  0x5f, 0x10, 0xe0, 0xe3, 0x5d, 0x30, 0xe0, 0xe3, 
  0x5c, 0x50, 0xe0, 0xe3, 0x5b, 0x70, 0xe0, 0xe3, 
  0xac, 0x02, 0x9f, 0xe5, 0xa4, 0x22, 0x9f, 0xe5, 
  0xa8, 0x42, 0x9f, 0xe5, 0xa8, 0x62, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x00, 0x30, 0x82, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x50, 0x84, 0xe5, 0x00, 0x50, 0x84, 0xe5, 
  0x00, 0x70, 0x86, 0xe5, 0x54, 0x02, 0x9f, 0xe5, 
  0x08, 0x00, 0x80, 0xe2, 0x5c, 0x12, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x6c, 0x02, 0x9f, 0xe5, 
  0x6c, 0x12, 0x9f, 0xe5, 0x00, 0x10, 0x80, 0xe5, 
  0x00, 0x10, 0xa0, 0xe3, 0x98, 0x02, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x94, 0x02, 0x9f, 0xe5, 
  0x00, 0x10, 0x80, 0xe5, 0x01, 0x09, 0xa0, 0xe3, 
  0x00, 0x10, 0xa0, 0xe3, 0x02, 0x26, 0xa0, 0xe3, 
  0x04, 0x30, 0xa0, 0xe3, 0x03, 0x10, 0x80, 0xe6, 
  0x02, 0x00, 0x50, 0xe1, 0xfc, 0xff, 0xff, 0x1a, 
  0x7f, 0x19, 0xa0, 0xe3, 0x1e, 0x30, 0xa0, 0xe3, 
  0x03, 0x3b, 0x83, 0xe2, 0x00, 0x00, 0xa0, 0xe3, 
  0x00, 0x2a, 0x83, 0xe1, 0x01, 0x00, 0x80, 0xe2, 
  0x14, 0x00, 0x50, 0xe3, 0x04, 0x20, 0x81, 0xe4, 
  0xfa, 0xff, 0xff, 0xba, 0x02, 0xe0, 0xa0, 0xe3, 
  0x03, 0xeb, 0x8e, 0xe2, 0x0e, 0x20, 0xa0, 0xe1, 
  0x18, 0x00, 0x50, 0xe3, 0x04, 0x00, 0x00, 0xaa, 
  0x00, 0xca, 0x82, 0xe1, 0x01, 0x00, 0x80, 0xe2, 
  0x18, 0x00, 0x50, 0xe3, 0x04, 0xc0, 0x81, 0xe4, 
  0xfa, 0xff, 0xff, 0xba, 0x40, 0x00, 0x50, 0xe3, 
  0x04, 0x00, 0x00, 0xaa, 0x00, 0x2a, 0x83, 0xe1, 
  0x01, 0x00, 0x80, 0xe2, 0x40, 0x00, 0x50, 0xe3, 
  0x04, 0x20, 0x81, 0xe4, 0xfa, 0xff, 0xff, 0xba, 
  0x01, 0x0a, 0x50, 0xe3, 0x04, 0x00, 0x00, 0xaa, 
  0x00, 0x2a, 0x8e, 0xe1, 0x01, 0x00, 0x80, 0xe2, 
  0x01, 0x0a, 0x50, 0xe3, 0x04, 0x20, 0x81, 0xe4, 
  0xfa, 0xff, 0xff, 0xba, 0x7f, 0x09, 0xa0, 0xe3, 
  0x03, 0x40, 0xa0, 0xe3, 0x10, 0x0f, 0x02, 0xee, 
  0x10, 0x4f, 0x03, 0xee, 0x10, 0x0f, 0x07, 0xee, 
  0x10, 0x0f, 0x05, 0xee, 0x3d, 0x00, 0xa0, 0xe3, 
  0x10, 0x0f, 0x01, 0xee, 0x01, 0x73, 0xa0, 0xe3, 
  0x04, 0x00, 0xa0, 0xe3, 0x05, 0x00, 0xc7, 0xe5, 
  0x01, 0x00, 0xa0, 0xe3, 0x06, 0x00, 0xc7, 0xe5, 
  0x01, 0x00, 0xa0, 0xe3, 0x07, 0x00, 0xc7, 0xe5, 
  0x03, 0x00, 0xa0, 0xe3, 0x08, 0x00, 0xc7, 0xe5, 
  0x00, 0x00, 0x40, 0x01, 0x12, 0x88, 0xd4, 0xf2, 
  0x01, 0x00, 0x00, 0xce, 0x01, 0xa0, 0xe3, 0x09, 
  0x00, 0xc7, 0xe5, 0x80, 0x00, 0xa0, 0xe3, 0x01, 
  0x00, 0xc7, 0xe5, 0x53, 0x00, 0xa0, 0xe3, 0x00, 
  0xf0, 0x29, 0xe1, 0x94, 0x11, 0x9f, 0xe5, 0x21, 
  0x03, 0xa0, 0xe3, 0x00, 0x10, 0x80, 0xe5, 0x10, 
  0x10, 0xa0, 0xe3, 0x22, 0x01, 0xa0, 0xe3, 0x00, 
  0x10, 0x80, 0xe5, 0x04, 0x00, 0x8f, 0xe2, 0x01, 
  0x00, 0x80, 0xe2, 0x10, 0xff, 0x2f, 0xe1, 0x5e, 
  0x4f, 0xb8, 0x78, 0x40, 0x1e, 0xfc, 0xd1, 0x38, 
  0x78, 0xf9, 0x7a, 0x09, 0x02, 0xbc, 0x7a, 0x21, 
  0x43, 0x09, 0x02, 0x7c, 0x7a, 0x21, 0x43, 0x09, 
  0x02, 0x3c, 0x7a, 0x21, 0x43, 0x11, 0x38, 0x01, 
  0x28, 0x2d, 0xdb, 0x03, 0x28, 0x2b, 0xdc, 0x80, 
  0x00, 0x55, 0x4c, 0x24, 0x58, 0xa7, 0x46, 0x46, 
  0x4b, 0x0e, 0x33, 0x7a, 0x7b, 0x12, 0x02, 0x3c, 
  0x7b, 0x22, 0x43, 0x12, 0xd0, 0x14, 0x1c, 0xf4, 
  0x3c, 0xff, 0x3c, 0x0e, 0xd5, 0x40, 0x07, 0x02, 
  0xd0, 0x1c, 0x1c, 0x0b, 0x1c, 0x21, 0x1c, 0x52, 
  0x1e, 0x8c, 0x5c, 0x9c, 0x54, 0xfb, 0xd1, 0x3b, 
  0xa4, 0x14, 0x23, 0x23, 0x60, 0xc0, 0x0f, 0x64, 
  0x30, 0x00, 0xe0, 0x66, 0x20, 0x78, 0x70, 0xcb, 
  0xe7, 0x01, 0xa2, 0x10, 0x47, 0x00, 0x00, 0x01, 
  0xf0, 0xa0, 0xe1, 0x04, 0x20, 0x8f, 0xe2, 0x01, 
  0x20, 0x82, 0xe2, 0x12, 0xff, 0x2f, 0xe1, 0x60, 
  0x20, 0xf0, 0xe7, 0x04, 0xe0, 0x4e, 0xe2, 0x00, 
  0x40, 0x2d, 0xe9, 0x00, 0xe0, 0x4f, 0xe1, 0x07, 
  0x40, 0x2d, 0xe9, 0xa8, 0x10, 0x8f, 0xe2, 0x00, 
  0x00, 0x91, 0xe5, 0x01, 0x00, 0x20, 0xe2, 0x00, 
  0x00, 0x81, 0xe5, 0x00, 0x00, 0x50, 0xe3, 0x03, 
  0x00, 0x00, 0x0a, 0x0d, 0x10, 0xa0, 0xe3, 0xcc, 
  0x20, 0x9f, 0xe5, 0x00, 0x10, 0x82, 0xe5, 0x02, 
  0x00, 0x00, 0xea, 0x0f, 0x10, 0xa0, 0xe3, 0xbc, 
  0x20, 0x9f, 0xe5, 0x00, 0x10, 0x82, 0xe5, 0x01, 
  0x00, 0xa0, 0xe3, 0x90, 0x10, 0x9f, 0xe5, 0x00, 
  0x00, 0x81, 0xe5, 0x6c, 0x10, 0x8f, 0xe2, 0x00, 
  0x00, 0x91, 0xe5, 0x01, 0x00, 0x40, 0xe2, 0x00, 
  0x00, 0x81, 0xe5, 0x00, 0x00, 0x50, 0xe3, 0x02, 
  0x00, 0x00, 0x1a, 0x94, 0x00, 0x9f, 0xe5, 0x94, 
  0x10, 0x9f, 0xe5, 0x00, 0x00, 0x81, 0xe5, 0x90, 
  0x20, 0x9f, 0xe5, 0x00, 0x10, 0x92, 0xe5, 0x07, 
  0x40, 0xbd, 0xe8, 0x0e, 0xf0, 0x69, 0xe1, 0x00, 
  0x80, 0xfd, 0xe8, 0x00, 0x00, 0x00, 0x8c, 0x20, 
  0x00, 0x00, 0x88, 0x50, 0x00, 0x00, 0x01, 0x02, 
  0x00, 0x00, 0x00, 0xf4, 0x00, 0x00, 0x00, 0x39, 
  0x00, 0x00, 0x00, 0x10, 0x40, 0x08, 0x00, 0x00, 
  0x40, 0x00, 0x00, 0x20, 0x40, 0x00, 0x00, 0x30, 
  0x4e, 0x00, 0x00, 0x00, 0x00, 0x00, 0xa0, 0x15, 
  0x80, 0x00, 0x00, 0x15, 0x2f, 0x8c, 0x9b, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x78, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x5d, 
  0x02, 0x00, 0x00, 0x5d, 0x02, 0x00, 0x00, 0x97, 
  0x02, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x40, 
  0x00, 0x40, 0x01, 0x64, 0x00, 0x50, 0x01, 0x70, 
  0x3e, 0x00, 0x00, 0xe0, 0x3c, 0x00, 0x00, 0x20, 
  0x00, 0x00, 0x90, 0x20, 0x00, 0x00, 0xa4, 0x85, 
  0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x78, 
  0x03, 0x00, 0x00, 0x58, 0x00, 0x50, 0x01, 0x70, 
  0xc3, 0x00, 0x00, 0x60, 0x00, 0x50, 0x01, 0x04, 
  0x00, 0x00, 0x84, 0x40, 0x01, 0x12, 0x08, 0x04, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x14
};

unsigned char pkt1[] = {
  0x00, 0x80, 0x02, 0x30, 0xec, 0xc0, 0x21, 0x01,
  0x00, 0x00, 0x28, 0x05, 0x06, 0x01, 0x0d, 0x17,
  0x6e, 0x0d, 0x03, 0x06, 0x11, 0x04, 0x06, 0x4e,
  0x13, 0x17, 0x01, 0x01, 0x17, 0xc9, 0xa0, 0x59,
  0xab, 0x4d, 0xac, 0xa3, 0xe4, 0x77, 0x21, 0xad,
  0x44, 0x40, 0x44, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00 
};

unsigned char pkt2[] = {
  0x00, 0x80, 0x02, 0x32, 0xec, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x2a, 0x8b, 0xd1, 0xb8, 0xdd
};

int main(int argc, char *argv[])
{
  int fd;
  int n;
  unsigned char buf[0x20000];

  fd = scan_dir("/proc/bus/usb",ST_VENDOR,ST_PRODUCT);
  if (fd < 0)
    {
      printf("Where is this crappy modem? huh!\n");
      return -1;
    }

  printf("Gotcha! fd=%d\n",fd);

  printf("PAGE_SIZE = %ld\n",PAGE_SIZE);
  fflush(stdout);

  /*n = myusb_bulk_msg(fd,0x5,data1,sizeof(data1),1000);*/
  n = myusb_bulk_msg(fd,0x7,pkt1,sizeof(pkt1),100);
  printf("write %d => %d\n",sizeof(data1),n);
  n = myusb_bulk_msg(fd,0x7,pkt2,sizeof(pkt2),100);
  printf("write %d => %d\n",sizeof(data1),n);
  n = USB_read(fd,buf,sizeof(buf));
  n = USB_read(fd,buf,sizeof(buf));

  close(fd);

  return 0;
}
