#ifndef USBFS_H
#define USBFS_H

/* Copyright Benoit PAPILLAULT (c) 2001

   this library is intented to be used over the usbdevfs file system,
   to access USB devices. It's a replacement for libusb, which has too much
   limitations for my work. And I wanted to investigate ioctl() and make
   some performance measurements with them.

   02/04/2001 : remove useless 'ep' parameter from usbfs_control_msg().
*/

#include <linux/usbdevice_fs.h>

/* search for the first available 'vendorID/productID' USB device
   and return an open descriptor to it. It returns -1 in case of failure.
   path is something similar to /proc/bus/usb.
*/

extern int usbfs_search(const char *path, int vendorID, int productID);

/* send a control message over the endpoint 'ep'. All remaining 
   parameters are used to filled the 'Setup' packet of the request,
   see $9.3 of USB 1.1 specs
*/

extern int usbfs_control_msg(int fd,int requesttype,int request,
			     int value,int idx,void * buf,int len,int timeout);

/* send or receive over a bulk endpoint. send or receive is indicated by
   the bit 7 of the endpoint (0=send, 1=received). The buffer may be as 
   large as you wanted
*/

extern int usbfs_bulk_msg(int fd,int ep,
			  unsigned char *buf,int size, int timeout);

/* reset an endpoint : what ? */

extern int usbfs_resetep(int fd,int ep);

/* clear a halt condition on an endpoint */

extern int usbfs_clear_halt(int fd, int ep);

/* usbfs_set_interface() is used to set alternate settings on an interface */

extern int usbfs_set_interface(int fd, int interface, int altsetting);

/* usbfs_set_configuration() is used to select a configuration.
   usually, this call is unneeded, since most devices have only one
   configuration */

extern int usbfs_set_configuration(int fd,int config);

/* a function to ease the work of filling URB structure used 
 for BULK transfer. sig must be 0 if unused. */

extern void usbfs_fill_bulk_urb(struct usbdevfs_urb * urb,
			       int ep, void *buf, size_t count, int sig);

/* usbfs_submit_urb() is used to send a URB (USB Request Block) down
   Answer to this request is get by usbfs_reapurb() */

extern int usbfs_submit_urb(int fd, struct usbdevfs_urb *urb);

/* purb is filled with the same pointer value, passed to usbfs_submit_urb() */

extern int usbfs_reapurb(int fd,struct usbdevfs_urb **purb);
extern int usbfs_reapurb_ndelay(int fd,struct usbdevfs_urb **purb);

/* usbfs_bulk() is an advanced function, still under study. It may be
 used to read/write on an endpoint */

extern int usbfs_bulk(int fd, int ep, void *buf, size_t count);
extern int usbfs_new_bulk(int fd, int ep, void *buf, size_t count);

/* kernel keeps tracks of who (what process or driver) is using a particular
   interface, since only one can use it at the same time. So to use an
   interface, your must first 'claim' it. Don't forget to release it
   after.
   
   'interface' is the number that appears after "If# " on 
   /proc/bus/usb/devices
*/

extern int usbfs_claimintf(int fd, int interface);
extern int usbfs_releaseintf(int fd, int interface);

/* usbfs_ioctl() is used to passed an ioctl command to the driver
   of the interface specified in ctrl.ifno. ctrl.ioctl_code & ctrl.data
   have the same usage than ioctl() param number 2 & 3
*/

extern int usbfs_ioctl(int fd,struct usbdevfs_ioctl * ctrl);

#endif
